<?php
namespace WPSecurityNinja\Plugin;

if ( ! function_exists( 'add_action' ) ) {
	die( 'Please don\'t open this file directly!' );
}


define( 'WF_SN_CS_SALT', 'monkey' );

class Wf_Sn_Cs {


	public static $hash_storage = 'https://api.wordpress.org/core/checksums/1.0/';

	// init plugin
	public static function init() {
		add_action( 'secnin_run_core_scanner', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'do_action_secnin_run_core_scanner' ) );

		if ( current_user_can( 'manage_options' ) ) {
			// add tab to Security Ninja tabs
			add_filter( 'sn_tabs', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'sn_tabs' ) );

			// enqueue scripts
			add_action( 'admin_enqueue_scripts', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'enqueue_scripts' ) );

			// register ajax endpoints
			add_action( 'wp_ajax_sn_core_get_file_source', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'get_file_source' ) );

			add_action( 'wp_ajax_sn_core_delete_file_do', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'delete_file' ) );

			add_action( 'init', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'schedule_cron_jobs' ) );

			add_action( 'wp_ajax_sn_core_restore_file_do', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'restore_file' ) );

			add_action( 'wp_ajax_sn_core_run_scan', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'do_action_core_run_scan' ) );

			add_action( 'wp_ajax_sn_core_delete_all_unknowns', array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'do_action_delete_all_unknowns' ) );

		}
	}


	/**
	 * do_action_secnin_run_core_scanner.
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0  Thursday, October 21st, 2021.
	 * @access  public static
	 * @global
	 * @return  void
	 */
	public static function do_action_secnin_run_core_scanner() {
		// Running the core scanner
		self::do_action_core_run_scan( true );
	}


	/**
	 * Schedule cron jobs
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Monday, December 21st, 2020.
	 * @version v1.0.1  Thursday, January 14th, 2021.
	 * @access  public static
	 * @return  void
	 */
	public static function schedule_cron_jobs() {
		if ( ! wp_next_scheduled( 'secnin_run_core_scanner' ) ) {
			wp_schedule_event( time(), 'daily', 'secnin_run_core_scanner' );
		}
	}





	/**
	 * Runs on admin_init action
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @return  void
	 */
	public static function do_action_delete_all_unknowns() {
    if ( ! check_ajax_referer( 'wf-cs-delete-all-unknown-nonce', false, false ) ) {
        wp_send_json_error( 'Invalid nonce' );
    }
    if ( ! current_user_can( 'manage_options' ) ) {
        wp_send_json_error(
            array(
                'message' => __( 'Failed.', 'security-ninja' ),
            )
        );
    }

    global $wp_filesystem;
    // Initialize the WordPress filesystem, if not already.
    if (empty($wp_filesystem)) {
        require_once(ABSPATH . 'wp-admin/includes/file.php');
        WP_Filesystem();
    }

    // DELETE ALL UNKNOWN FILES
    $results = get_option( 'wf_sn_cs_results' );
    if ( isset( $results['unknown_bad'] ) && is_array( $results['unknown_bad'] ) ) {
        $deleted_files = 0;
        foreach ( $results['unknown_bad'] as $ub ) {
            $filepath = ABSPATH . $ub;
            // Use WP filesystem method to delete files.
            if ($wp_filesystem->exists($filepath)) {
                if ($wp_filesystem->delete($filepath)) {
                    ++$deleted_files;
                } else {
                    // Handle error when file couldn't be deleted.
                    wp_send_json_error(
                        array(
                            'message' => sprintf( __( 'Failed to delete file: %s', 'security-ninja' ), $filepath ),
                        )
                    );
                }
            }
        }
        if ( $deleted_files > 0 ) {
            // translators: %d: Number of deleted files.
            $message = '<p>' . sprintf( esc_html__( 'Deleted %d unknown files in Core WordPress folders', 'security-ninja' ), $deleted_files ) . '</p>';
            wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner_delete_file', $message );
            $newresults = self::scan_files( true );
            if ( $newresults ) {
                update_option( 'wf_sn_cs_results', $newresults, false );
            }
            wp_send_json_success();
        }
    }
}




	/**
	 * Enqueue CSS and JS scripts
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @return  void
	 */
	public static function enqueue_scripts() {
		if ( wf_sn::is_plugin_page() ) {
			$plugin_url = plugin_dir_url( __FILE__ ); //@todo

			wp_enqueue_style( 'wp-jquery-ui-dialog' );
			wp_enqueue_script( 'jquery-ui-dialog' );

			wp_register_script( 'sn-core-js', $plugin_url . 'js/wf-sn-core-min.js', array( 'jquery' ), wf_sn::$version, true );

			$js_vars = array(
				'nonce'            => wp_create_nonce( 'wf_sn_cs' ),
				'run_scan_nonce'   => wp_create_nonce( 'wf-cs-run-scan-nonce' ),
				'delete_all_nonce' => wp_create_nonce( 'wf-cs-delete-all-unknown-nonce' ),
			);

			wp_localize_script( 'sn-core-js', 'wf_sn_cs', $js_vars );
			wp_enqueue_script( 'sn-core-js' );

			wp_enqueue_style( 'sn-core-css', $plugin_url . 'css/wf-sn-core.css', array(), wf_sn::$version );

			wp_enqueue_script( 'sn-core-snippet', $plugin_url . 'js/snippet.min.js', array(), wf_sn::$version, true );

			wp_enqueue_style( 'sn-core-snippet', $plugin_url . 'css/snippet.min.css', array(), wf_sn::$version );
		}
	}


	/**
	 * AJAX response for viewing file source
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @return  void
	 */
	public static function get_file_source() {
		check_ajax_referer( 'wf_sn_cs' );

		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Failed.', 'security-ninja' ),
				)
			);
		}

		$out = array();

		if ( ! isset( $_POST['filename'] ) ) {
			$error = new \WP_Error( '001', 'Filename not set' );
			wp_send_json_error( $error );
		}

		// check the checksum
		if ( md5( WF_SN_CS_SALT . stripslashes( $_POST['filename'] ) ) !== $_POST['hash'] ) {
			$error = new \WP_Error( '002', 'Cheating, eh?' );
			wp_send_json_error( $error );
		}

		$out['ext']    = pathinfo( $_POST['filename'], PATHINFO_EXTENSION );
		$out['source'] = '';

		if ( is_readable( $_POST['filename'] ) ) {
			$content = file_get_contents( $_POST['filename'] );
			if ( false !== $content ) {
				$out['err']    = 0;
				$out['source'] = wp_kses_post( $content );
			} else {
				$out['err'] = 'File is empty.';
			}
		} else {
			$out['err'] = 'File does not exist or is not readable.';
		}
		die( wp_json_encode( $out ) );
	}


	/**
	 * Returns the number of problems with files currently detected
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0  Thursday, October 21st, 2021.
	 * @access  private static
	 * @global
	 * @return  mixed
	 */
	private static function return_problem_count() {
		$results = get_option( 'wf_sn_cs_results' );

		$total = 0;
		if ( isset( $results['missing_bad'] ) ) {
			$total = $total + count( $results['missing_bad'] );
		}

		if ( isset( $results['changed_bad'] ) ) {
			$total = $total + count( $results['changed_bad'] );
		}

		if ( isset( $results['unknown_bad'] ) ) {
			$total = $total + count( $results['unknown_bad'] );
		}

		if ( $total > 0 ) {
			return $total;
		}

		return false;
	}

	/**
	 * Add this module tab
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @param   mixed   $tabs
	 * @return  mixed
	 */
	public static function sn_tabs( $tabs ) {
		$core_tab = array(
			'id'       => 'sn_core',
			'class'    => '',
			'label'    => 'Core Scanner',
			'callback' => array( __NAMESPACE__ . '\\Wf_Sn_Cs', 'core_page' ),
		);
		$done     = 0;
		$total    = count( $tabs );
		$problems = self::return_problem_count();
		if ( $problems ) {
			$core_tab['count'] = $problems;
		}

		for ( $i = 0; $i < $total; $i++ ) {
			if ( 'sn_core' === $tabs[ $i ]['id'] ) {
				$tabs[ $i ] = $core_tab;
				$done       = 1;
				break;
			}
		}

		if ( ! $done ) {
			$tabs[] = $core_tab;
		}

		return $tabs;
	}



	/**
	 *
	 * Generate a list of files to scan in a folder
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @param   mixed   $path
	 * @param   mixed   $extensions     Default: null
	 * @param   integer $depth          Default: 3
	 * @param   string  $relative_path  Default: ''
	 * @return  mixed
	 * @todo    - simililar function, taken from malware, perhaps put in core?
	 */
	public static function scan_folder( $path, $extensions = null, $depth = 3, $relative_path = '' ) {
		if ( ! is_dir( $path ) ) {
			return false;
		}

		if ( $extensions ) {
			$extensions  = (array) $extensions;
			$_extensions = implode( '|', $extensions );
		} else {

			$extensions  = array(); // empty array to find all types of files. array('php', 'php3', 'inc' );
			$_extensions = implode( '|', $extensions );
		}

		$relative_path = trailingslashit( $relative_path );
		if ( '/' === $relative_path ) {
			$relative_path = '';
		}

		$results = scandir( $path );
		$files   = array();

		foreach ( $results as $result ) {

			if ( '.' === $result[0] ) {
				continue;
			}

			if ( is_dir( $path . '/' . $result ) ) {
				if ( ! $depth || 'CVS' === $result ) {
					continue;
				}
				$found = self::scan_folder( $path . '/' . $result, $extensions, $depth - 1, $relative_path . $result );
				$files = array_merge_recursive( $files, $found );
			} elseif ( ! $extensions || preg_match( '~\.(' . $_extensions . ')$~', $result ) ) {
				$files[ $relative_path . $result ] = $path . '/' . $result;
			}
		}

		return $files;
	}




	/**
	 * Gets file hashes from wordpress.org API - stores in transient value
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @return  List of checksums from stored transient or API or false on error.
	 */
	public static function get_file_hashes() {
		$ver    = get_bloginfo( 'version' );
		$locale = get_locale();

		// Making sure we have the necessary functionality loaded.
		if ( ! function_exists( 'get_core_checksums' ) ) {
			include_once ABSPATH . 'wp-admin/includes/update.php';
		}

		$cs = get_core_checksums( $ver, isset( $locale ) ? $locale : 'en_US' );

		if ( empty( $cs['checksums'] ) ) {
			$details = array(
				'ver'    => $ver,
				'locale' => $locale,
			);
			wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner_update_hashes', sprintf( esc_html__( 'Could not get checksums with this locale, trying default checksums.', 'security-ninja' ) . ' v. ' . $ver . ' ' . $locale, $details ) );
			$cs = get_core_checksums( $ver, 'en_US' );
		}

		if ( $cs ) {
			$cleaned     = array();
			$themes_url  = trailingslashit( content_url( 'themes' ) );
			$plugins_url = trailingslashit( content_url( 'plugins' ) );

			$themes_path  = str_replace( site_url(), '', $themes_url );
			$plugins_path = str_replace( site_url(), '', $plugins_url );

			// Remove left trailing slash
			$themes_path  = ltrim( $themes_path, '/' );
			$plugins_path = ltrim( $plugins_path, '/' );

			foreach ( $cs as $path => $hash ) {
				if (
					strpos( $path, $themes_path ) !== false
					|| strpos( $path, $plugins_path ) !== false
					|| strpos( $path, '/plugins/akismet/' ) !== false
					|| strpos( $path, '/languages/themes/' ) !== false
				) {
				} else {
					$cleaned[ $path ] = $hash;
				}
			}
			$tmp = array(
				'version'   => $ver,
				'checksums' => $cleaned,
			);
			set_transient( 'wf_sn_hashes_' . $ver . '_' . $locale, $cleaned, MINUTE_IN_SECONDS * 15 ); // cached for 5 mins
			return $cleaned;
		}

		wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner_update_hashes', sprintf( esc_html__( 'There was a problem getting information about the WordPress original files.', 'security-ninja' ) . ' ' . $ver . ' ' . $locale ) );

		return false;
	}




	/**
	 * Find a match in an array from a list of needles
	 *
	 * ref: https://stackoverflow.com/questions/27816105/php-in-array-wildcard-match
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @param   mixed   $haystack
	 * @param   mixed   $needles
	 * @return  boolean
	 */
	public static function stripos_array( $haystack, $needles ) {
		foreach ( $needles as $needle ) {
			$res = stripos( $haystack, $needle );
			if ( false !== $res ) {
				return $res;
			}
		}
		return false;
	}







	/**
	 * Handles AJAX response - scan files and handles response
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @version v1.0.1  Monday, December 21st, 2020.
	 * @access  public static
	 * @return  mixed
	 */
	public static function do_action_core_run_scan( $internal = false ) {
		if ( ! $internal ) {
			check_ajax_referer( 'wf_sn_cs' );
		}

		// Check if the user has the required capability
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( 'You do not have sufficient permissions to access this page.', 'Permission Denied', array( 'response' => 403 ) );
		}

		$results = [];

		$doupdate = false;
		if ( isset( $_POST['doupdate'] ) && ( $_POST['doupdate'] ) ) {
			$doupdate = true;
		} else {
			$results = get_option( 'wf_sn_cs_results', array() );
		}

		// If we are doing an update with $internal set to true
		if ( $internal ) {
			$doupdate = true;
		}
		$start_time = microtime( true );

		if ( isset( $results ) && ! $doupdate ) {
			$spenttime           = microtime( true ) - $start_time;
			$results['run_time'] = number_format( $spenttime, 2 );
			wp_send_json_success( $results );
		}

		$results['missing_ok']  = array();
		$results['changed_ok']  = array();
		$results['missing_bad'] = array();
		$results['changed_bad'] = array();
		$results['unknown_bad'] = array();
		$results['ok']          = array();
		$results['last_run']    = current_time( 'timestamp' );
		$results['total']       = 0;
		$results['run_time']    = 0;

		$i = 0;

		$ver = get_bloginfo( 'version' );

		// Files ok to be missing
		$missing_ok = array( 'index.php', 'readme.html', 'license.txt', 'wp-config-sample.php', 'wp-admin/install.php', 'wp-admin/upgrade.php', 'wp-config.php', 'plugins/hello.php', 'licens.html', '/languages/plugins/akismet-' );

		// Files ok to be modified
		$changed_ok = array( 'index.php', 'wp-config.php', 'wp-config-sample.php', 'readme.html', 'license.txt', 'wp-includes/version.php' );

		$filehashes = self::get_file_hashes();

		if ( $filehashes ) {

			// ** Checking for unknown files
			$files     = self::scan_folder( ABSPATH . WPINC, null, 9, WPINC );
			$all_files = $files;

			$files     = self::scan_folder( ABSPATH . 'wp-admin', null, 9, 'wp-admin' );
			$all_files = array_merge( $all_files, $files );

			foreach ( $all_files as $key => $af ) {
				if ( ! isset( $filehashes[ $key ] ) ) {
					$results['unknown_bad'][] = $key;
				}
			}

			// Checking if core has been modified
			$results['total'] = count( $filehashes );

			foreach ( $filehashes as $file => $hash ) {
				clearstatcache();

				if ( file_exists( ABSPATH . $file ) ) {
					if ( md5_file( ABSPATH . $file ) === $hash ) {
					} elseif ( in_array( $file, $changed_ok, true ) ) {
						$results['changed_ok'][] = $file;
					} else {
						$results['changed_bad'][] = $file;
					}
				} elseif ( in_array( $file, $missing_ok, true ) ) {
						$results['missing_ok'][] = $file;
				} else {
					$results['missing_bad'][] = $file;
				}
			} // foreach file

			do_action( 'security_ninja_core_scanner_done_scanning', $results, microtime( true ) - $start_time );

			$results['out'] = '';

			if ( isset( $results['last_run'] ) && $results['last_run'] ) {
				$allisgood       = true;
				$results['out'] .= '<div id="sn-cs-results">';

				if ( $results['unknown_bad'] ) {
					$allisgood       = false;
					$results['out'] .= '<div class="sn-cs-changed-bad">';

					$results['out'] .= '<div class="core-title"><h4>' . __( 'The following files are unknown and should not be in your core folders', 'security-ninja' ) . '</h4></div>';

					$results['out'] .= '<div class="changedcont"><p class="description">' . __( 'These are files not included with WordPress default installation and should not be in your core WordPress folders.', 'security-ninja' ) . '</p>';

					$results['out'] .= '<p class="description">' . __( 'These files can be leftovers from older WordPress installations, and are no longer needed.', 'security-ninja' ) . '</p>';

					$results['out'] .= self::list_files( $results['unknown_bad'], false, false, true );
					$results['out'] .= ' <button href="#delete-all-dialog" class="sn-delete-all-files button button-primary button-small">' . __( 'Delete all', 'security-ninja' ) . '</button>';
					$results['out'] .= '</div></div>';
				}

				if ( $results['changed_bad'] ) {
					$allisgood       = false;
					$results['out'] .= '<div class="sn-cs-changed-bad"><div class="core-title">';

					$results['out'] .= '<h4>' . __( 'The following WordPress core files have been modified', 'security-ninja' ) . '</h4>';
					$results['out'] .= '</div>';

					$results['out'] .= '<div class="changedcont">';
					$results['out'] .= '<p>' . __( 'If you did not modify the following files yourself, this could indicate an infection on your website.', 'security-ninja' ) . '</p>';
					$results['out'] .= self::list_files( $results['changed_bad'], false, true );

					$results['out'] .= '</div></div>';
				}

				if ( $results['missing_bad'] ) {
					$allisgood       = false;
					$results['out'] .= '<div class="sn-cs-missing-bad"><div class="core-title">';

					$results['out'] .= '<h4>' . __( 'Following core files are missing.', 'security-ninja' ) . '</h4>';
					$results['out'] .= '</div>';

					$results['out'] .= '<div class="changedcont">';

					$results['out'] .= '<p class="description">' . esc_html__( 'Missing core files might indicate a bad auto-update or they simply were not copied on the server when the site was setup.', 'security-ninja' ) . '</p>';

					$results['out'] .= '<p class="description">' . esc_html__( 'If there is no legitimate reason for the files to be missing use the restore action to create them.', 'security-ninja' ) . '</p>';

					$results['out'] .= self::list_files( $results['missing_bad'], false, true );

					$results['out'] .= '</div></div>';
				}

				if ( $allisgood ) {
					$results['out'] .= '<p>' . __( 'No problems found' ) . '</p>';
				}
				$results['out'] .= '</div><!-- #sn-cs-results -->';
			}

			if ( isset( $results['missing_ok'] ) ) {
				unset( $results['missing_ok'] );
			}
			if ( isset( $results['changed_ok'] ) ) {
				unset( $results['changed_ok'] );
			}

			// No need storing the OK files.
			if ( isset( $results['ok'] ) ) {
				unset( $results['ok'] );
			}

			if ( $start_time ) {
				$spenttime           = microtime( true ) - $start_time;
				$results['run_time'] = number_format( $spenttime, 2 );
			}

			$version = get_bloginfo( 'version' );
			$locale  = get_locale();

			$results['last_scan'] = sprintf(
				// translators:
				esc_html__( 'Last scan at %1$s. %2$s files were checked in %3$s sec. WordPress version %4$s %5$s', 'security-ninja' ),
				gmdate( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $results['last_run'] ),
				number_format( $results['total'] ),
				number_format( $results['run_time'], 2 ),
				$version,
				$locale
			);

			update_option( 'wf_sn_cs_results', $results, false );
			wp_send_json_success( $results );
		} else {
			wp_send_json_error( 'error - woopsie!' );
		}
	}


	/**
	 * Do the actual scanning of the files
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @param   boolean $return Default: false
	 * @return  mixed
	 */
	public static function scan_files( $return = false ) {
		$local_time   = current_datetime();
		$current_time = $local_time->getTimestamp() + $local_time->getOffset();
		// No nonce check, can be run via scheduled scanner also
		$results['missing_ok']  = array();
		$results['changed_ok']  = array();
		$results['missing_bad'] = array();
		$results['changed_bad'] = array();
		$results['unknown_bad'] = array();
		$results['ok']          = array();
		$results['last_run']    = $current_time;
		$results['total']       = 0;
		$results['run_time']    = 0;
		$start_time             = microtime( true );

		$i = 0;

		$ver = get_bloginfo( 'version' );

		// Files ok to be missing
		$missing_ok = array( 'index.php', 'readme.html', 'license.txt', 'wp-config-sample.php', 'wp-admin/install.php', 'wp-admin/upgrade.php', 'wp-config.php', 'plugins/hello.php', 'licens.html', '/languages/plugins/akismet-' );

		// Files ok to be modified
		$changed_ok = array( 'index.php', 'wp-config.php', 'wp-config-sample.php', 'readme.html', 'license.txt', 'wp-includes/version.php' );

		$filehashes = self::get_file_hashes();

		if ( $filehashes ) {

			// ** Checking for unknown files
			$files     = self::scan_folder( ABSPATH . WPINC, null, 9, WPINC );
			$all_files = $files;

			$files     = self::scan_folder( ABSPATH . 'wp-admin', null, 9, 'wp-admin' );
			$all_files = array_merge( $all_files, $files );

			foreach ( $all_files as $key => $af ) {
				if ( ! isset( $filehashes[ $key ] ) ) {
					$results['unknown_bad'][] = $key;
				}
			}

			// Checking if core has been modified
			$results['total'] = count( $filehashes );

			foreach ( $filehashes as $file => $hash ) {
				clearstatcache();

				if ( file_exists( ABSPATH . $file ) ) {
					if ( $hash === md5_file( ABSPATH . $file ) ) {

					} elseif ( in_array( $file, $changed_ok, true ) ) {
						$results['changed_ok'][] = $file;
					} else {
						$results['changed_bad'][] = $file;
					}
				} elseif ( in_array( $file, $missing_ok, true ) ) {
						$results['missing_ok'][] = $file;
				} else {
					$results['missing_bad'][] = $file;
				}
			}

			do_action( 'security_ninja_core_scanner_done_scanning', $results, microtime( true ) - $start_time );
			$results['run_time'] = microtime( true ) - $start_time;

			if ( $return ) {
				return $results;
			} else {
				update_option( 'wf_sn_cs_results', $results, false );
				die( '1' );
			}
		} else {
			// no file definitions for this version of WP
			if ( $return ) {
				return null;
			} else {
				update_option( 'wf_sn_cs_results', null, false );
				die( '0' );
			}
		}
	}


	/**
	 * The page displayed in the tabs
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @return  void
	 */
	public static function core_page() {

		?>
		<div class="submit-test-container card">

			<h3><?php echo esc_html__( 'Scan core WordPress files and folders', 'security-ninja' ); ?></h3>
			<?php

			global $secnin_fs;

			?>
			<p><?php esc_html_e( 'Check for modified files in WordPress itself and detect extra files that should not be there.', 'security-ninja' ); ?></p>

			<?php
			echo '<input type="button" value="' . esc_html__( 'Scan core files', 'security-ninja' ) . '" id="sn-run-core-scan" class="button button-primary button-hero" />';

			?>
		</div>
		<div id="wf-sn-core-scanner-response" class="card">
			<p class="spinner"></p>
			<p id="last_scan"></p>
		</div>

		<?php

		// dialogs
		echo '<div id="source-dialog" style="display: none;" title="' . esc_html__( 'File Source', 'security-ninja' ) . '"><p>' . esc_html__( 'Please wait.', 'security-ninja' ) . '</p></div>';
		echo '<div id="restore-dialog" style="display: none;" title="' . esc_html__( 'Restore File', 'security-ninja' ) . '"><p>' . esc_html__( 'Please wait.', 'security-ninja' ) . '</p></div>';
		echo '<div id="delete-dialog" style="display: none;" title="' . esc_html__( 'Delete file', 'security-ninja' ) . '"><p>' . esc_html__( 'Please wait.', 'security-ninja' ) . '</p></div>';
	}


	/**
	 * Check if files can be restored
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @return  mixed
	 */
	public static function check_file_write() {
		// @todo lars check this out
		$url = wp_nonce_url( 'options.php?page=wf-sn', 'wf-sn-cs' );
		ob_start();
		$creds = request_filesystem_credentials( $url, '', false, false, null );
		ob_end_clean();
		return (bool) $creds;
	}


	/**
	 * Restore the a file - AJAX
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @return  void
	 */
	public static function restore_file() {
		check_ajax_referer( 'wf_sn_cs' );
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( 'You do not have sufficient permissions to access this page.', 'Permission Denied', array( 'response' => 403 ) );
		}

		$file = str_replace( ABSPATH, '', stripslashes( $_POST['filename'] ) );

		$url   = wp_nonce_url( 'options.php?page=wf-sn', 'wf-sn-cs' );
		$creds = request_filesystem_credentials( $url, '', false, false, null );
		if ( ! WP_Filesystem( $creds ) ) {
			die( 'can\'t write to file.' );
		}

		$org_file = wp_remote_get( 'https://core.trac.wordpress.org/browser/tags/' . get_bloginfo( 'version' ) . '/src/' . $file . '?format=txt' );
		if ( ! $org_file['body'] ) {
			die( 'can\'t download remote file source.' );
		}

		global $wp_filesystem;
		if ( ! $wp_filesystem->put_contents( trailingslashit( ABSPATH ) . $file, $org_file['body'], FS_CHMOD_FILE ) ) {
			die( 'unknown error while writing file.' );
		}

		self::scan_files();
		wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner', 'Restored file', array( 'name' => $file ) );
		wp_send_json_success();
	}



	/**
	 * Delete a file - AJAX call
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @version v1.0.1  Monday, February 6th, 2023.
	 * @version v1.0.2  Friday, November 17th, 2023.
	 * @access  public static
	 * @return  void
	 */
	public static function delete_file() {
		check_ajax_referer( 'wf_sn_cs' );
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Failed.', 'security-ninja' ),
				)
			);
		}
		$file  = str_replace( ABSPATH, '', stripslashes( $_POST['filename'] ) );
		$url   = wp_nonce_url( 'options.php?page=wf-sn', 'wf-sn-cs' );
		$creds = request_filesystem_credentials( $url, '', false, false, null );
		if ( ! WP_Filesystem( $creds ) ) {
			// translators: Problems deleting a specific file
			wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner_delete_file', sprintf( esc_html__( 'Cannot delete %s', 'security-ninja' ), $file ) );
			// translators: Problems deleting a specific file
			die( sprintf( esc_html__( 'Cannot delete %s', 'security-ninja' ), $file ) );
		}

		global $wp_filesystem;
		if ( ! $wp_filesystem->delete( trailingslashit( ABSPATH ) . $file, false ) ) {
			// translators: Problem deleting specific file
			wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner_delete_file', sprintf( esc_html__( 'Unknown error deleting %s', 'security-ninja' ), $file ) );
			// translators: Problem deleting specific file
			die( sprintf( esc_html__( 'Unknown error deleting %s', 'security-ninja' ), esc_html( $file ) ) );
		} else {
			// translators: A file was deleted by a user
			wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner_delete_file', sprintf( esc_html__( 'File %s was deleted by user.', 'security-ninja' ), $file ) );

		}
		wf_sn_el_modules::log_event( 'security_ninja', 'core_scanner', 'Deleted file', array( 'name' => $file ) );
		wp_send_json_success();
	}

	/**
	 * Helper function for listing files
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @param   mixed   $files
	 * @param   boolean $view       Show view button. Default false.
	 * @param   boolean $restore    Show restore button. Default false.
	 * @param   boolean $delete     Show delete button. Default false.
	 * @return  string
	 */
	public static function list_files( $files, $view = false, $restore = false, $delete = false ) {
		$out  = '';
		$out .= '<ul class="sn-file-list">';

		foreach ( $files as $file ) {
			$out .= '<li>';
			$out .= '<span class="sn-file">' . $file . '</span>';
			if ( $view ) {
				$out .= ' <button data-hash="' . md5( WF_SN_CS_SALT . ABSPATH . $file ) . '" data-file="' . ABSPATH . $file . '" href="#source-dialog" class="sn-show-source button button-small">' . __( 'View', 'security-ninja' ) . '</button>';
			}
			if ( $restore ) {
				$out .= ' <button data-hash="' . md5( WF_SN_CS_SALT . ABSPATH . $file ) . '" data-file-short="' . $file . '" data-file="' . ABSPATH . $file . '" href="#restore-dialog" class="sn-restore-source button button-small">' . __( 'Restore', 'security-ninja' ) . '</button>';
			}

			if ( $delete ) {
				$out .= ' <button data-hash="' . md5( WF_SN_CS_SALT . ABSPATH . $file ) . '" data-file-short="' . $file . '" data-file="' . ABSPATH . $file . '" href="#delete-dialog" class="sn-delete-source button button-small">' . __( 'Delete', 'security-ninja' ) . '</abutton>';
			}

			$out .= '</li>';
		}

		$out .= '</ul>';

		return $out;
	}

	/**
	 * Clean-up when deactivated
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  public static
	 * @return  void
	 */
	public static function deactivate() {
		$centraloptions = Wf_Sn::get_options();
		if ( ! isset( $centraloptions['remove_settings_deactivate'] ) ) {
			return;
		}
		if ( $centraloptions['remove_settings_deactivate'] ) {
			wp_clear_scheduled_hook( 'secnin_run_core_scanner' );
			delete_option( 'wf_sn_cs_results' );
		}
	}
}

add_action( 'plugins_loaded', array( __NAMESPACE__ . '\wf_sn_cs', 'init' ) );
register_deactivation_hook( WF_SN_BASE_FILE, array( __NAMESPACE__ . '\wf_sn_cs', 'deactivate' ) );