<?php

namespace WPSecurityNinja\Plugin;

/**
 * Wizard
 * Based on Whizzie package, customized heavily.
 *
 * @author  Catapult Themes
 * @author  WP Security Ninja
 * @since   v0.0.1
 * @version v1.0.0  Wednesday, March 2nd, 2022.
 * @global
 */
class Wf_Sn_Wiz {



	protected static $version = '1.2.0';

	/** @var string Wizard page slug and title. */
	protected static $page_slug  = 'security-ninja-wizard';
	protected static $page_title = 'Security Ninja Wizard';

	/** @var array Wizard steps set by user. */
	protected static $config_steps = array();

	/**
	 * Relative plugin url for this plugin folder
	 * @since 1.0.0
	 * @var string
	 */
	protected static $plugin_url = '';

	/**
	 * welcome_screen_page.
	 *
	 * @author   Lars Koudal
	 * @since    v0.0.1
	 * @version  v1.0.0  Wednesday, December 23rd, 2020.
	 * @access   public static
	 * @return   void
	 */
	public static function welcome_screen_page() {

		$plugin_name = 'Security Ninja';
		if ( secnin_fs()->is__premium_only() ) {
			if ( secnin_fs()->can_use_premium_code() ) {
				// If whitelabel enabled
				if ( class_exists( __NAMESPACE__ . '\wf_sn_wl' ) ) {
					if ( wf_sn_wl::is_active() ) {
						$plugin_name = wf_sn_wl::get_new_name();
					}
				}
			}
		}

		add_submenu_page( 'wf-sn', $plugin_name . ' Install Wizard', 'Get started', 'manage_options', 'security-ninja-wizard', array( __NAMESPACE__ . '\\Wf_Sn_Wiz', 'welcome_page' ) );
	}


	/**
	 * str_ends_with.
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, March 2nd, 2022.
	 * @access  public static
	 * @param   mixed   $haystack
	 * @param   mixed   $needle
	 * @return  integer
	 */
	public static function str_ends_with( $haystack, $needle ) {
		if ( '' === $haystack && '' !== $needle ) {
			return false;
		}
		$len = strlen( $needle );
		return 0 === substr_compare( $haystack, $needle, -$len, $len );
	}



	/**
	 * Hooks and filters
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, March 2nd, 2022.
	 * @access  public static
	 * @return  void
	 */
	public static function init() {
		add_action( 'wp_ajax_secnin_activate_firewall', array( __NAMESPACE__ . '\\Wf_Sn_Wiz', 'activate_firewall' ) );

		add_action( 'wp_ajax_secnin_activate_default_fixes', array( __NAMESPACE__ . '\\Wf_Sn_Wiz', 'activate_default_fixes' ) );

		add_action( 'wp_ajax_secnin_finish_security_tests', array( __NAMESPACE__ . '\\Wf_Sn_Wiz', 'finish_security_tests' ) );

		add_action( 'admin_enqueue_scripts', array( __NAMESPACE__ . '\\Wf_Sn_Wiz', 'enqueue_scripts' ) );
	}

	/**
	 * Make an interface for the wizard
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, March 2nd, 2022.
	 * @access  public static
	 * @return  void
	 */
	public static function wizard_page() {
		$url = wp_nonce_url( add_query_arg( array( 'plugins' => 'go' ) ), 'whizzie-setup' );

		// copied from TGM
		$method = ''; // Leave blank so WP_Filesystem can populate it as necessary.
		$fields = array_keys( $_POST ); // Extra fields to pass to WP_Filesystem.
		$creds  = request_filesystem_credentials( esc_url_raw( $url ), $method, false, false, $fields );
		if ( false === ( $creds ) ) {
			return true; // Stop the normal page form from displaying, credential request form will be shown.
		}
		// Now we have some credentials, setup WP_Filesystem.
		if ( ! WP_Filesystem( $creds ) ) {
			// Our credentials were no good, ask the user for them again.
			request_filesystem_credentials( esc_url_raw( $url ), $method, true, false, $fields );
			return true;
		}
		/* If we arrive here, we have the filesystem */ ?>

		<div class="wrap">
			<?php
			wf_sn::show_topbar();

			echo '<div class="card whizzie-wrap">';
			// The wizard is a list with only one item visible at a time
			$steps = self::get_steps();
			echo '<ul class="whizzie-menu">';
			$allowed_html = wp_kses_allowed_html( 'post' );

			foreach ( $steps as $step ) {
				$class = 'step step-' . esc_attr( $step['id'] );
				echo '<li data-step="' . esc_attr( $step['id'] ) . '" class="' . esc_attr( $class ) . '">';
				printf( '<h2>%s</h2>', esc_html( $step['title'] ) );
				// $content is split into summary and detail
				$content = call_user_func( array( __NAMESPACE__ . '\Wf_Sn_Wiz', $step['view'] ) );

			
				if ( isset( $content['summary'] ) ) {
					printf(
						'<div class="summary">%s</div>',
						wp_kses( $content['summary'], $allowed_html )
					);
				}
				// if ( isset( $step['detail'] ) ) {
				// 	// Add a link to see more detail
				// 	printf( '<p><a href="#" class="more-info">%s</a></p>', __( 'More Info', 'security-ninja' ) );
				// 	printf(
				// 		'<div class="detail">%s</div>',
				// 		$content['detail'] // Need to escape this @todo
				// 	);
				// }

				// Show newsletter
				if ( isset( $step['newsletter'] ) && ( $step['newsletter'] ) ) {
					$current_user    = wp_get_current_user();
					$show_newsletter = true;

					if ( secnin_fs()->is__premium_only() ) {
						if ( secnin_fs()->can_use_premium_code() ) {
							// If whitelabel enabled
							if ( class_exists( __NAMESPACE__ . '\wf_sn_wl' ) ) {
								if ( wf_sn_wl::is_active() ) {
									$show_newsletter = false;
								}
							}
						}
					}

					if ( $show_newsletter ) {
						echo '<div class="wizardnewsletter">
					<h3>Join the newsletter</h3>
	
					<h4>Updates about the plugin and important WordPress security in general</h4>';

						echo '<form class="ml-block-form" action="https://assets.mailerlite.com/jsonp/16490/forms/106309154087372203/subscribe" data-code="" method="post" target="_blank">';
						echo '<table>
							<tbody>
								<tr>
									<td>
										<input type="text" class="regular-text" data-inputmask="" name="fields[name]" placeholder="Name" autocomplete="name" style="width:15em;" value="' . esc_html( $current_user->display_name ) . '">
									</td>
									<td>
										<input aria-label="email" aria-required="true" data-inputmask="" type="email" class="regular-text required email" data-inputmask="" name="fields[email]" placeholder="Email" autocomplete="email" style="width:15em;" value="' . esc_html( $current_user->user_email ) . '" required="required">
									</td>
									<td>
									<button type="submit" class="button button-primary button-small">Subscribe</button>
								</td>
								</tr>
						</table>';

						echo '<input type="hidden" name="fields[signupsource]" value="Plugin Wizard v.' . esc_attr( Wf_Sn::get_plugin_version() ) . '">
						<input type="hidden" name="ml-submit" value="1">
						<input type="hidden" name="anticsrf" value="true">
					</form>';

						echo '<p>You can unsubscribe anytime. For more details, review our <a href="https://wpsecurityninja.com/privacy-policy/" target="_blank" rel="noopener">Privacy Policy</a>.</p>
				</div>';
					}
				}

				// The next button
				if ( isset( $step['button_text'] ) && $step['button_text'] ) {
					$button_class = '';
					if ( isset( $step['button_class'] ) ) {
						$button_class = $step['button_class'];
					}
					printf(
						'<div class="button-wrap"><a href="#" class="button button-hero button-primary do-it %s" data-callback="%s" data-step="%s">%s</a></div>',
						esc_attr( $button_class ),
						esc_attr( $step['callback'] ),
						esc_attr( $step['id'] ),
						esc_html( $step['button_text'] )
					);
				}
				// The skip button
				if ( isset( $step['can_skip'] ) && $step['can_skip'] ) {
					printf(
						'<div class="button-wrap" style="margin-left: 0.5em;"><a href="#" class="button button-secondary button-hero do-it" data-callback="%s" data-step="%s">%s</a></div>',
						'do_next_step',
						esc_attr( $step['id'] ),
						__( 'Skip', 'security-ninja' )
					);
				}
				echo '</li>';
			}
			echo '</ul>';
			echo '<hr>';
			echo '<ul class="whizzie-nav">';

			foreach ( $steps as $step ) {
				if ( isset( $step['icon'] ) && $step['icon'] ) {
					if ( isset( $step['skip_icon'] ) && $step['skip_icon'] ) {
						$iconout = '';
					} else {
						$iconout = '<span class="dashicons dashicons-' . esc_attr( $step['icon'] ) . '"></span>';
					}
					echo '<li class="nav-step-' . esc_attr( $step['id'] ) . '">' . $iconout . '<span class="titleshort">' . esc_html( $step['title_short'] ) . '</span></li>';
				}
			}
			echo '</ul>';
			?>
			<div class="step-loading"><span class="spinner"></span></div>
		</div><!-- .whizzie-wrap -->
		<div class="secnin-wizard-warning">
			<p><small>Warning: Rerunning this wizard will overwrite your old settings.</small></p>
		</div>
		</div><!-- .wrap -->
		<?php
	}



	/**
	 * Set options for the steps
	 * Incorporate any options set by the theme dev
	 * Return the array for the steps
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, March 2nd, 2022.
	 * @access  public static
	 * @return  mixed
	 */
	public static function get_steps() {
		$dev_steps = self::$config_steps;

		$plugin_name = 'Security Ninja';
		if ( secnin_fs()->is__premium_only() ) {
			if ( secnin_fs()->can_use_premium_code() ) {
				// If whitelabel enabled
				if ( class_exists( __NAMESPACE__ . '\wf_sn_wl' ) ) {
					if ( wf_sn_wl::is_active() ) {
						$plugin_name = wf_sn_wl::get_new_name();
					}
				}
			}
		}

		$steps = array(
			'intro'          => array(
				'id'           => 'intro',
				'detail' => '###TEST###',
				'title'        => __( 'Thank you for choosing', 'security-ninja' ) . ' ' . esc_attr( $plugin_name ) . ' &hearts;',
				'title_short'  => 'Welcome',
				'icon'         => 'dashboard',
				'view'         => 'get_step_intro', // Callback for content
				'callback'     => 'do_next_step', // Callback for JS
				'button_text'  => 'Start Wizard',
				'button_class' => ' button-hero', // extra CSS - optional
				'can_skip'     => false, // Show a skip button?
			),
			'firewall'       => array(
				'id'          => 'firewall',
				'title'       => __( 'Activate Firewall Protection', 'security-ninja' ),
				'title_short' => 'Firewall',
				'icon'        => 'shield',
				'view'        => 'get_step_firewall',
				'callback'    => 'activate_firewall',
				'button_text' => __( 'Activate', 'security-ninja' ),
				'can_skip'    => true,
			),
			'default_fixes'  => array(
				'id'          => 'default_fixes',
				'title'       => __( 'Activate Default Security Measures', 'security-ninja' ),
				'title_short' => 'Fixes',
				'icon'        => 'admin-plugins',
				'view'        => 'get_step_default_fixes',
				'callback'    => 'activate_default_fixes',
				'button_text' => 'Activate',
				'can_skip'    => true,
			),
			'security_tests' => array(
				'id'          => 'security_tests',
				'title'       => __( 'Thank you - All done ', 'security-ninja' ) . ' &hearts;',
				'title_short' => 'Done',
				'icon'        => 'yes-alt',
				'view'        => 'get_step_security_tests',
				'newsletter'  => true,
				'can_skip'    => false,
			),
		);

		// Iterate through each step and replace with dev config values
		if ( $dev_steps ) {
			// Configurable elements - these are the only ones the dev can update from config.php
			$can_config = array( 'title', 'icon', 'button_text', 'can_skip' );
			foreach ( $dev_steps as $dev_step ) {
				// We can only proceed if an ID exists and matches one of our IDs
				if ( isset( $dev_step['id'] ) ) {
					$id = $dev_step['id'];
					if ( isset( $steps[ $id ] ) ) {
						foreach ( $can_config as $element ) {
							if ( isset( $dev_step[ $element ] ) ) {
								$steps[ $id ][ $element ] = $dev_step[ $element ];
							}
						}
					}
				}
			}
		}
		return $steps;
	}


	/**
	 * Print the content for the intro step
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, February 3rd, 2021.
	 * @access  public static
	 * @return  mixed
	 */
	public static function get_step_intro() {
		$content = array();

		$direct_link = admin_url( 'admin.php?page=wf-sn' );

		$content['summary'] = '<h3>Use this wizard to get you up and running in a few minutes.</h3>';

		$content['summary'] .= '<p>This wizard sets up standard best security practices for your website.</p>';

		$content['summary'] .= '<p>Click the button to get started or <a href="' . esc_url( $direct_link ) . '">click here to go to the dashboard</a>.';
		return $content;
	}


	/**
	 * get_step_default_fixes.
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, February 3rd, 2021.
	 * @access  public static
	 * @return  mixed
	 */
	public static function get_step_default_fixes() {
		$content            = array();
		$content['summary'] = '<p>Additional protection measures for your website.</p>';

		$content['summary'] .= "<p>Enhance your website's security with our comprehensive suite of default measures designed to protect against common vulnerabilities and ensure your site's integrity.</p><p>This includes automatic updates for plugins, concealment of version information and development tools from your site's code, and the removal of unneeded files to keep your site clean and secure. Additionally, we strengthen your site's defenses with Secure Cookies and a selection of crucial security headers to safeguard against various types of cyber threats.</p>";

		$url = wf_sn::generate_sn_web_link('install_wizard', 'docs/installation-and-usage/activate-default-security-measures/');

		if ($url) {
			$content['summary'] .= '<p><a href="' . esc_url($url) . '" target="_blank" rel="noopener">Learn more about the default security measures</a></p>';
		}

		return $content;
	}


	/**
	 * Get the content for the plugins step
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Wednesday, February 3rd, 2021.
	 * @access  public static
	 * @return  mixed
	 */
	public static function get_step_firewall() {
		$content = array();

		$content['summary'] = '<p>The firewall protects you against hack attempts, blocks known spammers and malicious IPs.</p>';

		$content['summary'] .= '<ul class="summarylist"><li>Enable the Cloud Firewall list of 600 million+ bad IPs. The list is updated daily from many sources.</li>
        <li>Block suspicious queries (probing for security holes)</li>
		<li>Protect your login form from mass attacks</li>		
        <li>Enable the Block IP network - automatically block IPs trying to hack other websites</li></ul>';

		$content['summary'] .= '<div class="attbox">In case you ever get locked out - Save this URL:</br>';
		$unblock_url         = wf_sn_cf::get_unblock_url();
		$content['summary'] .= '<pre>' . esc_url( $unblock_url ) . '</pre>';
		$content['summary'] .= 'Visiting that URL will unblock your IP and allow you to log in.</br></div>';

		return $content;
	}


	/**
	 * get_step_security_tests.
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Thursday, February 4th, 2021.
	 * @access  public static
	 * @return  mixed
	 */
	public static function get_step_security_tests() {

		$current_user = wp_get_current_user();

		$content = array();

		$content['summary'] = '<p>Great, you have now finished the getting started wizard!</p>';

		$content['summary'] .= '<p>Your website is now protected, but there are many more features to secure your website even further.</p>';

		$content['summary'] .= '<p>Visit our <a href="https://wpsecurityninja.com/docs/" target="_blank" rel="noopener">documentation</a> or <a href="https://wpsecurityninja.com/help/" target="_blank" rel="noopener">reach out to our support</a> if you have any problems.</p>';

		$secninlink = admin_url( 'admin.php?page=wf-sn' );

		$content['summary'] .= '<p><a href="' . esc_url( $secninlink ) . '" class="button button-primary">Finish Wizard</a></p>';

		return $content;
	}



	/**
	 * Enqueue CSS and JS scripts
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Friday, December 18th, 2020.
	 * @access  static
	 * @return  void
	 */
	public static function enqueue_scripts() {
		global $current_screen;
		$needle = 'page_security-ninja-wizard';

		// Checks if we are on the wizard page, otherwise we leave
		if ( ! self::str_ends_with( $current_screen->id, $needle ) ) {
			return;
		}

		wp_register_script( 'secnin-wizard', WF_SN_PLUGIN_URL . 'modules/wizard/assets/js/secnin-wizard-min.js', array( 'jquery' ), wf_sn::$version, true );

		$js_vars = array(
			'nonce'         => wp_create_nonce( 'secnin_wizard_nonce' ),
			'sn_plugin_url' => WF_SN_PLUGIN_URL,
		);

		// If the test class is loaded
		if ( class_exists( __NAMESPACE__ . '\wf_sn_tests' ) ) {
			$js_vars['security_tests'] = wf_sn_tests::return_security_tests();
		}

		wp_localize_script( 'secnin-wizard', 'whizzie_params', $js_vars ); // @todo

		wp_enqueue_script( 'secnin-wizard' );
	}


	/**
	 * Print the content for the final step
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Thursday, February 4th, 2021.
	 * @access  public static
	 * @return  mixed
	 */
	public static function get_step_done() {
		$content = array();
		// The summary element will be the content visible to the user
		$content['summary'] = '<h3>All done!</h3>';

		$direct_link = admin_url( 'admin.php?page=wf-sn' );

		$content['summary'] .= '<p>Click the button to get started or <a href="' . esc_url( $direct_link ) . '" class="button button-hero button-primary">Security Ninja</a></p>';

		return $content;
	}


	/**
	 * Activates the firewall and sets default settings
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Sunday, January 31st, 2021.
	 * @access  public static
	 * @return  void
	 */
	public static function activate_firewall() {
		check_ajax_referer( 'secnin_wizard_nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Failed.', 'security-ninja' ),
				)
			);
		}

		$default_firewall_options = wf_sn_cf::get_options();
		// Tweak the default settings
		$default_firewall_options['active']            = 1;
		$default_firewall_options['global']            = 1;
		$default_firewall_options['filterqueries']     = 1;
		$default_firewall_options['usecloud']          = 1;
		$default_firewall_options['globalbannetwork']  = 1;
		$default_firewall_options['blockadminlogin']   = 0;
		$default_firewall_options['trackvisits']       = 0;
		$default_firewall_options['blocked_countries'] = '';

		update_option( WF_SN_CF_OPTIONS_KEY, $default_firewall_options, false );

		if ( class_exists( __NAMESPACE__ . '\Wf_sn_cf' ) ) {
			// Sends password to current user admin email address.
			$current_user = wp_get_current_user();
			Wf_sn_cf::send_secret_access_unblock_url( $current_user->user_email );
		}

		$results = array(
			'done'    => 1,
			'message' => 'Firewall enabled.',
		);

		wp_send_json_success( $results );
		exit;
	}








	/**
	 * The "Tests" tab -> final step before done page.
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Sunday, January 31st, 2021.
	 * @access  public static
	 * @return  void
	 */
	public static function finish_security_tests() {
		check_ajax_referer( 'secnin_wizard_nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Failed.', 'security-ninja' ),
				)
			);
		}

		$results = array(
			'done'    => 1,
			'message' => 'Security Tests done',
		);

		wp_send_json_success( $results );
		exit;
	}


	/**
	 *
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Sunday, January 31st, 2021.
	 * @access  public static
	 * @return  void
	 */
	public static function all_done() {
		check_ajax_referer( 'secnin_wizard_nonce' );

		$results = array(
			'done'    => 1,
			'message' => 'All finished.',
		);
		wp_send_json_success( $results );
	}


	/**
	 *
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0  Sunday, January 31st, 2021.
	 * @access  public static
	 * @return  void
	 */
	public static function activate_default_fixes() {
		check_ajax_referer( 'secnin_wizard_nonce' );

		if ( ! current_user_can( 'manage_options' ) ) {
			wp_send_json_error(
				array(
					'message' => __( 'Failed.', 'security-ninja' ),
				)
			);
		}

		$default_fixes_options = Wf_Sn_Fixes::get_options();
		// Tweak the default settings
		$default_fixes_options['disable_username_enumeration'] = 1;
		$default_fixes_options['hide_wp']                      = 1;
		$default_fixes_options['hide_wlw']                     = 1;
		$default_fixes_options['hide_php_ver']                 = 1;
		$default_fixes_options['hide_wp_debug']                = 1;
		$default_fixes_options['application_passwords']        = 1;
		$default_fixes_options['remove_unwanted_files']        = 1;

		$default_fixes_options['enable_xcto']  = 1;
		$default_fixes_options['sechead_xcto'] = 'X-Content-Type-Options: nosniff';

		$default_fixes_options['enable_xfo']  = 1;
		$default_fixes_options['sechead_xfo'] = 'X-Frame-Options: SAMEORIGIN';

		$default_fixes_options['enable_sts']  = 1;
		$default_fixes_options['sechead_sts'] = 'Strict-Transport-Security: max-age=31536000;';

		$default_fixes_options['enable_rp']  = 1;
		$default_fixes_options['sechead_rp'] = 'Referrer-Policy: same-origin';

		$default_fixes_options['secure_cookies'] = 1;

		update_option( WF_SN_FIXES_OPTIONS_KEY, $default_fixes_options, false );

		$auto_updates = (array) get_site_option( 'auto_update_plugins', array() );
		// Enables automatic background updates
		$auto_updates[] = 'security-ninja/security-ninja.php';
		$auto_updates[] = 'security-ninja-premium/security-ninja.php';

		update_site_option( 'auto_update_plugins', $auto_updates );

		$results = array(
			'done'    => 1,
			'message' => 'Default fixes enabled.',
		);

		wp_send_json_success( $results );
		exit;
	}
}
// hook everything up
add_action( 'plugins_loaded', array( __NAMESPACE__ . '\Wf_Sn_Wiz', 'init' ) );
